@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP          #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
@@ -37,7 +38,9 @@
 
 module Diagrams.Animation.Active where
 
+#if __GLASGOW_HASKELL__ < 710
 import           Control.Applicative (pure, (<$>))
+#endif
 
 import           Diagrams.Core
 import           Diagrams.TrailLike
@@ -45,6 +48,7 @@ import           Diagrams.TrailLike
 import           Data.Active
 
 type instance V (Active a) = V a
+type instance N (Active a) = N a
 
 -- Yes, these are all orphan instances. Get over it.  We don't want to
 -- put them in the 'active' package because 'active' is supposed to be
@@ -95,5 +99,5 @@ instance Juxtaposable a => Juxtaposable (Active a) where
       )
       a1
 
---instance Alignable a => Alignable (Active a) where
---  alignBy v d a = alignBy v d <$> a
+-- instance Alignable a => Alignable (Active a) where
+--   alignBy v d a = alignBy v d <$> a
