@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE Rank2Types            #-}
@@ -28,41 +27,35 @@ module Diagrams.Combinators
        , atop
        , beneath
        , beside
+       , atDirection
 
          -- * n-ary operations
        , appends
-       , position, decorateTrail, decorateLocatedTrail, decoratePath
+       , position, atPoints
        , cat, cat'
        , CatOpts(_catMethod, _sep), catMethod, sep
        , CatMethod(..)
 
        ) where
 
-import           Data.Typeable
-
-import           Control.Lens          (Lens', generateSignatures,
-                                        lensRules, makeLensesWith, (%~), (&),
-                                        (.~), (^.), _Wrapping)
-import           Data.AdditiveGroup
-import           Data.AffineSpace      ((.+^))
+import           Control.Lens          hiding (beside, ( # ))
 import           Data.Default.Class
 import           Data.Monoid.Deletable (toDeletable)
 import           Data.Monoid.MList     (inj)
-#if __GLASGOW_HASKELL__ < 707
 import           Data.Proxy
-#endif
 import           Data.Semigroup
 import qualified Data.Tree.DUAL        as D
-import           Data.VectorSpace
 
 import           Diagrams.Core
 import           Diagrams.Core.Types   (QDiagram (QD))
-import           Diagrams.Located
-import           Diagrams.Path
+import           Diagrams.Direction
 import           Diagrams.Segment      (straight)
-import           Diagrams.Trail        (Trail, trailVertices)
 import           Diagrams.Util
 
+import           Linear.Affine
+import           Linear.Metric
+import           Linear.Vector
+
 ------------------------------------------------------------
 -- Working with envelopes
 ------------------------------------------------------------
@@ -76,23 +69,23 @@ import           Diagrams.Util
 --   >     circle 1 # fc green
 --   >     |||
 --   >     (    c # dashingG [0.1,0.1] 0 # lc white
---   >       <> square 2 # withEnvelope (c :: D R2) # fc blue
+--   >       <> square 2 # withEnvelope (c :: D V2 Double) # fc blue
 --   >     )
 --   > c = circle 0.8
 --   > withEnvelopeEx = sqNewEnv # centerXY # pad 1.5
-withEnvelope :: (HasLinearMap (V a), Enveloped a, Monoid' m)
-           => a -> QDiagram b (V a) m -> QDiagram b (V a) m
+withEnvelope :: (InSpace v n a, Metric v, OrderedField n, Monoid' m, Enveloped a)
+           => a -> QDiagram b v n m -> QDiagram b v n m
 withEnvelope = setEnvelope . getEnvelope
 
 -- | Use the trace from some object as the trace for a diagram, in
 --   place of the diagram's default trace.
-withTrace :: (HasLinearMap (V a), Traced a, OrderedField (Scalar (V a)), InnerSpace (V a), Monoid' m)
-          => a -> QDiagram b (V a) m -> QDiagram b (V a) m
+withTrace :: (InSpace v n a, Metric v, OrderedField n, Monoid' m, Traced a)
+          => a -> QDiagram b v n m -> QDiagram b v n m
 withTrace = setTrace . getTrace
 
 -- | @phantom x@ produces a \"phantom\" diagram, which has the same
 --   envelope and trace as @x@ but produces no output.
-phantom :: (Backend b (V a), Typeable (V a), Enveloped a, Traced a, Monoid' m) => a -> QDiagram b (V a) m
+phantom :: (InSpace v n a, Monoid' m, Enveloped a, Traced a) => a -> QDiagram b v n m
 phantom a = QD $ D.leafU ((inj . toDeletable . getEnvelope $ a) <> (inj . toDeletable . getTrace $ a))
 
 -- | @pad s@ \"pads\" a diagram, expanding its envelope by a factor of
@@ -101,21 +94,17 @@ phantom a = QD $ D.leafU ((inj . toDeletable . getEnvelope $ a) <> (inj . toDele
 --   origin, so if the origin is not centered the padding may appear
 --   \"uneven\".  If this is not desired, the origin can be centered
 --   (using, e.g., 'centerXY' for 2D diagrams) before applying @pad@.
-pad :: ( Backend b v
-       , InnerSpace v, OrderedField (Scalar v)
-       , Monoid' m )
-    => Scalar v -> QDiagram b v m -> QDiagram b v m
+pad :: (Metric v, OrderedField n, Monoid' m)
+    => n -> QDiagram b v n m -> QDiagram b v n m
 pad s d = withEnvelope (d # scale s) d
 
 -- | @frame s@ increases the envelope of a diagram by and absolute amount @s@,
 --   s is in the local units of the diagram. This function is similar to @pad@,
 --   only it takes an absolute quantity and pre-centering should not be
 --   necessary.
-frame :: ( Backend b v, InnerSpace v, OrderedField (Scalar v), Monoid' m)
-        => Scalar v -> QDiagram b v m -> QDiagram b v m
-frame s d = setEnvelope (onEnvelope t (d^.envelope)) d
-  where
-    t f = \x -> f x + s
+frame :: (Metric v, OrderedField n, Monoid' m)
+        => n -> QDiagram b v n m -> QDiagram b v n m
+frame s = over envelope (onEnvelope $ \f x -> f x + s)
 
 -- | @strut v@ is a diagram which produces no output, but with respect
 --   to alignment and envelope acts like a 1-dimensional segment
@@ -128,12 +117,8 @@ frame s d = setEnvelope (onEnvelope t (d^.envelope)) d
 --   <<diagrams/src_Diagrams_Combinators_strutEx.svg#diagram=strutEx&width=300>>
 --
 --   > strutEx = (circle 1 ||| strut unitX ||| circle 1) # centerXY # pad 1.1
-strut :: ( Backend b v, Typeable v
-         , InnerSpace v
-         , OrderedField (Scalar v)
-         , Monoid' m
-         )
-      => v -> QDiagram b v m
+strut :: (Metric v, OrderedField n, Monoid' m)
+      => v n -> QDiagram b v n m
 strut v = QD $ D.leafU (inj . toDeletable $ env)
   where env = translate ((-0.5) *^ v) . getEnvelope $ straight v
   -- note we can't use 'phantom' here because it tries to construct a
@@ -154,9 +139,8 @@ strut v = QD $ D.leafU (inj . toDeletable $ env)
 --   the cosine of the difference in angle, and leaving it unchanged
 --   when this factor is negative.
 extrudeEnvelope
-  :: ( Ord (Scalar v), Num (Scalar v), AdditiveGroup (Scalar v)
-     , Floating (Scalar v), HasLinearMap v, InnerSpace v, Monoid' m )
-  => v -> QDiagram b v m -> QDiagram b v m
+  :: (Metric v, OrderedField n, Monoid' m)
+  => v n -> QDiagram b v n m -> QDiagram b v n m
 extrudeEnvelope = deformEnvelope 0.5
 
 -- | @intrudeEnvelope v d@ asymmetrically \"intrudes\" the envelope of
@@ -167,24 +151,22 @@ extrudeEnvelope = deformEnvelope 0.5
 --   Note that this could create strange inverted envelopes, where
 --   @ diameter v d < 0 @.
 intrudeEnvelope
-  :: ( Ord (Scalar v), Num (Scalar v), AdditiveGroup (Scalar v)
-     , Floating (Scalar v), HasLinearMap v, InnerSpace v, Monoid' m )
-  => v -> QDiagram b v m -> QDiagram b v m
+  :: (Metric v, OrderedField n, Monoid' m)
+  => v n -> QDiagram b v n m -> QDiagram b v n m
 intrudeEnvelope = deformEnvelope (-0.5)
 
 -- Utility for extrudeEnvelope / intrudeEnvelope
 deformEnvelope
-  :: ( Ord (Scalar v), Num (Scalar v), AdditiveGroup (Scalar v)
-     , Floating (Scalar v), HasLinearMap v, InnerSpace v, Monoid' m )
-  => (Scalar v) -> v -> QDiagram b v m -> QDiagram b v m
-deformEnvelope s v d = setEnvelope (getEnvelope d & _Wrapping Envelope %~ deformE) d
+  :: (Metric v, OrderedField n, Monoid' m)
+  => n -> v n -> QDiagram b v n m -> QDiagram b v n m
+deformEnvelope s v = over (envelope . _Wrapping Envelope) deformE
   where
     deformE = Option . fmap deformE' . getOption
     deformE' env v'
-        | dot > 0 = Max $ getMax (env v') + (dot * s) / magnitude v'
+        | dp > 0 = Max $ getMax (env v') + (dp * s) / norm v'
         | otherwise = env v'
       where
-        dot = v' <.> v
+        dp = v' `dot` v
 
 ------------------------------------------------------------
 -- Combining two objects
@@ -193,8 +175,8 @@ deformEnvelope s v d = setEnvelope (getEnvelope d & _Wrapping Envelope %~ deform
 -- | @beneath@ is just a convenient synonym for @'flip' 'atop'@; that is,
 --   @d1 \`beneath\` d2@ is the diagram with @d2@ superimposed on top of
 --   @d1@.
-beneath :: (HasLinearMap v, OrderedField (Scalar v), InnerSpace v, Monoid' m)
-     => QDiagram b v m -> QDiagram b v m -> QDiagram b v m
+beneath :: (Metric v, OrderedField n, Monoid' m)
+     => QDiagram b v n m -> QDiagram b v n m -> QDiagram b v n m
 beneath = flip atop
 
 infixl 6 `beneath`
@@ -234,9 +216,18 @@ infixl 6 `beneath`
 --   To get something like @beside v x1 x2@ whose local origin is
 --   identified with that of @x2@ instead of @x1@, use @beside
 --   (negateV v) x2 x1@.
-beside :: (Juxtaposable a, Semigroup a) => V a -> a -> a -> a
+beside :: (Juxtaposable a, Semigroup a) => Vn a -> a -> a -> a
 beside v d1 d2 = d1 <> juxtapose v d1 d2
 
+-- | Place two diagrams (or other juxtaposable objects) adjacent to
+--   one another, with the second diagram placed in the direction 'd'
+--   from the first.  The local origin of the resulting combined
+--   diagram is the same as the local origin of the first.  See the
+--   documentation of 'beside' for more information.
+atDirection :: (InSpace v n a, Metric v, Floating n, Juxtaposable a, Semigroup a)
+            => Direction v n -> a -> a -> a
+atDirection = beside . fromDirection
+
 ------------------------------------------------------------
 -- Combining multiple objects
 ------------------------------------------------------------
@@ -251,7 +242,7 @@ beside v d1 d2 = d1 <> juxtapose v d1 d2
 --   > appendsEx = appends c (zip (iterateN 6 (rotateBy (1/6)) unitX) (repeat c))
 --   >             # centerXY # pad 1.1
 --   >   where c = circle 1
-appends :: (Juxtaposable a, Monoid' a) => a -> [(V a,a)] -> a
+appends :: (Juxtaposable a, Monoid' a) => a -> [(Vn a,a)] -> a
 appends d1 apps = d1 <> mconcat (map (\(v,d) -> juxtapose v d1 d) apps)
 
 -- | Position things absolutely: combine a list of objects
@@ -260,39 +251,17 @@ appends d1 apps = d1 <> mconcat (map (\(v,d) -> juxtapose v d1 d) apps)
 --
 --   <<diagrams/src_Diagrams_Combinators_positionEx.svg#diagram=positionEx&height=300>>
 --
---   > positionEx = position (zip (map mkPoint [-3, -2.8 .. 3]) (repeat dot))
---   >   where dot       = circle 0.2 # fc black
---   >         mkPoint x = p2 (x,x^2)
-position :: (HasOrigin a, Monoid' a) => [(Point (V a), a)] -> a
+--   > positionEx = position (zip (map mkPoint [-3, -2.8 .. 3]) (repeat spot))
+--   >   where spot      = circle 0.2 # fc black
+--   >         mkPoint :: Double -> P2 Double
+--   >         mkPoint x = p2 (x,x*x)
+position :: (InSpace v n a, HasOrigin a, Monoid' a) => [(Point v n, a)] -> a
 position = mconcat . map (uncurry moveTo)
 
--- | Combine a list of diagrams (or paths) by using them to
---   \"decorate\" a trail, placing the local origin of one object at
---   each successive vertex of the trail.  The first vertex of the
---   trail is placed at the origin.  If the trail and list of objects
---   have different lengths, the extra tail of the longer one is
---   ignored.
-decorateTrail :: (InnerSpace (V a), OrderedField (Scalar (V a)), HasOrigin a, Monoid' a)
-              => Trail (V a) -> [a] -> a
-decorateTrail = decorateLocatedTrail . (`at` origin)
-
--- | Combine a list of diagrams (or paths) by using them to
---   \"decorate\" a concretely located trail, placing the local origin
---   of one object at each successive vertex of the trail. If the
---   trail and list of objects have different lengths, the extra tail
---   of the longer one is ignored.
-decorateLocatedTrail :: (InnerSpace (V a), OrderedField (Scalar (V a)), HasOrigin a, Monoid' a)
-              => Located (Trail (V a)) -> [a] -> a
-decorateLocatedTrail t = position . zip (trailVertices t)
-
--- | Combine a list of diagrams (or paths) by using them to
---   \"decorate\" a path, placing the local origin of one object at
---   each successive vertex of the path.  If the path and list of objects
---   have different lengths, the extra tail of the longer one is
---   ignored.
-decoratePath :: (InnerSpace (V a), OrderedField (Scalar (V a)), HasOrigin a, Monoid' a)
-             => Path (V a) -> [a] -> a
-decoratePath p = position . zip (concat $ pathVertices p)
+-- | Curried version of @position@, takes a list of points and a list of
+--   objects.
+atPoints :: (InSpace v n a, HasOrigin a, Monoid' a) => [Point v n] -> [a] -> a
+atPoints ps as = position $ zip ps as
 
 -- | Methods for concatenating diagrams.
 data CatMethod = Cat     -- ^ Normal catenation: simply put diagrams
@@ -311,9 +280,9 @@ data CatMethod = Cat     -- ^ Normal catenation: simply put diagrams
                          --   of separation, diagrams may overlap.
 
 -- | Options for 'cat''.
-data CatOpts v = CatOpts { _catMethod       :: CatMethod
-                         , _sep             :: Scalar v
-                         , catOptsvProxy__ :: Proxy v
+data CatOpts n = CatOpts { _catMethod    :: CatMethod
+                         , _sep          :: n
+                         , catOptsvProxy :: Proxy n
                          }
 
 -- The reason the proxy field is necessary is that without it,
@@ -327,24 +296,22 @@ data CatOpts v = CatOpts { _catMethod       :: CatMethod
 -- this is not a problem when using the 'sep' lens, as its type is
 -- more restricted.
 
-makeLensesWith
-  ( lensRules & generateSignatures .~ False)
-  ''CatOpts
+makeLensesWith (lensRules & generateSignatures .~ False) ''CatOpts
 
 -- | Which 'CatMethod' should be used:
 --   normal catenation (default), or distribution?
-catMethod :: forall v. Lens' (CatOpts v) CatMethod
+catMethod :: Lens' (CatOpts n) CatMethod
 
 -- | How much separation should be used between successive diagrams
 --   (default: 0)?  When @catMethod = Cat@, this is the distance between
 --   /envelopes/; when @catMethod = Distrib@, this is the distance
 --   between /origins/.
-sep :: forall v. Lens' (CatOpts v) (Scalar v)
+sep :: Lens' (CatOpts n) n
 
-instance Num (Scalar v) => Default (CatOpts v) where
-  def = CatOpts { _catMethod       = Cat
-                , _sep             = 0
-                , catOptsvProxy__ = Proxy
+instance Num n => Default (CatOpts n) where
+  def = CatOpts { _catMethod    = Cat
+                , _sep          = 0
+                , catOptsvProxy = Proxy
                 }
 
 -- | @cat v@ positions a list of objects so that their local origins
@@ -355,10 +322,8 @@ instance Num (Scalar v) => Default (CatOpts v) where
 --
 --   See also 'cat'', which takes an extra options record allowing
 --   certain aspects of the operation to be tweaked.
-cat :: ( Juxtaposable a, Monoid' a, HasOrigin a
-       , InnerSpace (V a), OrderedField (Scalar (V a))
-       )
-       => V a -> [a] -> a
+cat :: (InSpace v n a, Metric v, Floating n, Juxtaposable a, Monoid' a, HasOrigin a)
+       => v n -> [a] -> a
 cat v = cat' v def
 
 -- | Like 'cat', but taking an extra 'CatOpts' arguments allowing the
@@ -378,13 +343,11 @@ cat v = cat' v def
 --   Note that @cat' v (with & catMethod .~ Distrib) === mconcat@
 --   (distributing with a separation of 0 is the same as
 --   superimposing).
-cat' :: ( Juxtaposable a, Monoid' a, HasOrigin a
-        , InnerSpace (V a), OrderedField (Scalar (V a))
-        )
-     => V a -> CatOpts (V a) -> [a] -> a
+cat' :: (InSpace v n a, Metric v, Floating n, Juxtaposable a, Monoid' a, HasOrigin a)
+     => v n -> CatOpts n -> [a] -> a
 cat' v (CatOpts { _catMethod = Cat, _sep = s }) = foldB comb mempty
   where comb d1 d2 = d1 <> (juxtapose v d1 d2 # moveOriginBy vs)
-        vs = s *^ normalized (negateV v)
+        vs = s *^ signorm (negated v)
 
 cat' v (CatOpts { _catMethod = Distrib, _sep = s }) =
-  position . zip (iterate (.+^ (s *^ normalized v)) origin)
+  position . zip (iterate (.+^ (s *^ signorm v)) origin)
