@@ -1,6 +1,3 @@
-{-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE TypeFamilies     #-}
-
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.Points
@@ -9,28 +6,33 @@
 -- Maintainer  :  diagrams-discuss@googlegroups.com
 --
 -- Points in space.  For more tools for working with points and
--- vectors, see "Data.AffineSpace" and "Diagrams.Coordinates".
+-- vectors, see "Linear.Affine".
 --
 -----------------------------------------------------------------------------
 
 module Diagrams.Points
        ( -- * Points
-         Point, origin, (*.)
+         Point (..), origin, (*.)
 
          -- * Point-related utilities
        , centroid
        , pointDiagram
-
+       , _Point, lensP
        ) where
 
-import           Diagrams.Core          (pointDiagram)
+import           Diagrams.Core        (pointDiagram)
 import           Diagrams.Core.Points
 
-import           Control.Arrow          ((&&&))
+import           Data.Foldable        as F
 
-import           Data.AffineSpace.Point
-import           Data.VectorSpace
+import           Linear.Affine
+import           Linear.Vector
 
 -- | The centroid of a set of /n/ points is their sum divided by /n/.
-centroid :: (VectorSpace v, Fractional (Scalar v)) => [Point v] -> Point v
-centroid = P . uncurry (^/) . (sumV &&& (fromIntegral . length)) . map unPoint
+centroid :: (Additive v, Fractional n) => [Point v n] -> Point v n
+centroid = meanV
+
+meanV :: (Foldable f, Additive v, Fractional a) => f (v a) -> v a
+meanV = uncurry (^/) . F.foldl' (\(s,c) e -> (e ^+^ s,c+1)) (zero,0)
+{-# INLINE meanV #-}
+
