@@ -4,8 +4,10 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE Rank2Types                 #-}
+{-# LANGUAGE StandaloneDeriving         #-}
 {-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
+{-# LANGUAGE UndecidableInstances       #-}
 {-# LANGUAGE ViewPatterns               #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 -----------------------------------------------------------------------------
@@ -25,14 +27,16 @@
 module Diagrams.TwoD.Path
        ( -- * Constructing path-based diagrams
 
-         stroke, stroke', strokeTrail, strokeT, strokeTrail', strokeT'
+         stroke, stroke'
+       , strokePath, strokeP, strokePath', strokeP'
+       , strokeTrail, strokeT, strokeTrail', strokeT'
        , strokeLine, strokeLoop
        , strokeLocTrail, strokeLocT, strokeLocLine, strokeLocLoop
 
          -- ** Stroke options
 
        , FillRule(..)
-       , FillRuleA(..), getFillRule, fillRule
+       , getFillRule, fillRule, _fillRule
        , StrokeOpts(..), vertexNames, queryFillRule
 
          -- ** Inside/outside testing
@@ -41,35 +45,42 @@ module Diagrams.TwoD.Path
 
          -- * Clipping
 
-       , Clip(..), clipBy, clipTo, clipped
+       , Clip(..), _Clip, _clip
+       , clipBy, clipTo, clipped
+
+         -- * Intersections
+
+       , intersectPoints, intersectPoints'
+       , intersectPointsP, intersectPointsP'
+       , intersectPointsT, intersectPointsT'
        ) where
 
-import           Control.Applicative   (liftA2)
-import           Control.Lens          (Lens, Lens', generateSignatures,
-                                        lensRules, makeLensesWith, makeWrapped,
-                                        op, (.~), (^.), _Wrapped')
-import qualified Data.Foldable         as F
+import           Control.Applicative       (liftA2)
+import           Control.Lens              hiding (at, transform)
+import qualified Data.Foldable             as F
 import           Data.Semigroup
 import           Data.Typeable
 
-import           Data.AffineSpace
 import           Data.Default.Class
-import           Data.VectorSpace
 
-import           Diagrams.Combinators  (withEnvelope, withTrace)
-import           Diagrams.Coordinates
+import           Diagrams.Angle
+import           Diagrams.Combinators      (withEnvelope, withTrace)
 import           Diagrams.Core
 import           Diagrams.Core.Trace
-import           Diagrams.Located      (Located, mapLoc, unLoc)
+import           Diagrams.Located          (Located, mapLoc, unLoc)
 import           Diagrams.Parametric
 import           Diagrams.Path
 import           Diagrams.Segment
-import           Diagrams.Solve
+import           Diagrams.Solve.Polynomial
 import           Diagrams.Trail
 import           Diagrams.TrailLike
-import           Diagrams.TwoD.Segment ()
+import           Diagrams.TwoD.Segment
 import           Diagrams.TwoD.Types
-import           Diagrams.Util         (tau)
+import           Diagrams.TwoD.Vector
+import           Diagrams.Util             (tau)
+
+import           Linear.Affine
+import           Linear.Vector
 
 ------------------------------------------------------------
 --  Trail and path traces  ---------------------------------
@@ -79,14 +90,14 @@ import           Diagrams.Util         (tau)
 
 -- XXX can the efficiency of this be improved?  See the comment in
 -- Diagrams.Path on the Enveloped instance for Trail.
-instance Traced (Trail R2) where
+instance RealFloat n => Traced (Trail V2 n) where
   getTrace = withLine $
       foldr
-        (\seg bds -> moveOriginBy (negateV . atEnd $ seg) bds <> getTrace seg)
+        (\seg bds -> moveOriginBy (negated . atEnd $ seg) bds <> getTrace seg)
         mempty
     . lineSegments
 
-instance Traced (Path R2) where
+instance RealFloat n => Traced (Path V2 n) where
   getTrace = F.foldMap getTrace . op Path
 
 ------------------------------------------------------------
@@ -95,7 +106,7 @@ instance Traced (Path R2) where
 
 -- | Enumeration of algorithms or \"rules\" for determining which
 --   points lie in the interior of a (possibly self-intersecting)
---   closed path.
+--   path.
 data FillRule = Winding  -- ^ Interior points are those with a nonzero
                          --   /winding/ /number/.  See
                          --   <http://en.wikipedia.org/wiki/Nonzero-rule>.
@@ -104,7 +115,11 @@ data FillRule = Winding  -- ^ Interior points are those with a nonzero
                          --   direction crosses the path an odd number
                          --   of times. See
                          --   <http://en.wikipedia.org/wiki/Even-odd_rule>.
-    deriving (Eq, Show)
+    deriving (Show, Typeable, Eq, Ord)
+
+instance AttributeClass FillRule
+instance Semigroup FillRule where
+  _ <> b = b
 
 instance Default FillRule where
   def = Winding
@@ -147,35 +162,47 @@ instance Default (StrokeOpts a) where
         , _queryFillRule = def
         }
 
--- | Convert a path into a diagram.  The resulting diagram has the
+-- | Convert a 'ToPath' object into a diagram.  The resulting diagram has the
 --   names 0, 1, ... assigned to each of the path's vertices.
 --
 --   See also 'stroke'', which takes an extra options record allowing
---   its behavior to be customized.
---
---   Note that a bug in GHC 7.0.1 causes a context stack overflow when
---   inferring the type of @stroke@.  The solution is to give a type
---   signature to expressions involving @stroke@, or (recommended)
---   upgrade GHC (the bug is fixed in 7.0.2 onwards).
-stroke :: Renderable (Path R2) b
-       => Path R2 -> Diagram b R2
-stroke = stroke' (def :: StrokeOpts ())
-
-instance Renderable (Path R2) b => TrailLike (QDiagram b R2 Any) where
-  trailLike = stroke . trailLike
+--   its behaviour to be customized.
+stroke :: (InSpace V2 n t, ToPath t, TypeableFloat n, Renderable (Path V2 n) b)
+       => t -> QDiagram b V2 n Any
+stroke = strokeP . toPath
 
 -- | A variant of 'stroke' that takes an extra record of options to
---   customize its behavior.  In particular:
+--   customize its behaviour.  In particular:
 --
 --     * Names can be assigned to the path's vertices
 --
 --   'StrokeOpts' is an instance of 'Default', so @stroke' ('with' &
 --   ... )@ syntax may be used.
-stroke' :: (Renderable (Path R2) b, IsName a) => StrokeOpts a -> Path R2 -> Diagram b R2
-stroke' opts path
-  | null (pLines ^. _Wrapped') =           mkP pLoops
+stroke' :: (InSpace V2 n t, ToPath t, TypeableFloat n, Renderable (Path V2 n) b, IsName a)
+       => StrokeOpts a -> t -> QDiagram b V2 n Any
+stroke' opts = strokeP' opts . toPath
+
+-- | 'stroke' specialised to 'Path'.
+strokeP :: (TypeableFloat n, Renderable (Path V2 n) b)
+        => Path V2 n -> QDiagram b V2 n Any
+strokeP = strokeP' (def :: StrokeOpts ())
+
+-- | 'stroke' specialised to 'Path'.
+strokePath :: (TypeableFloat n, Renderable (Path V2 n) b)
+        => Path V2 n -> QDiagram b V2 n Any
+strokePath = strokeP
+
+instance (TypeableFloat n, Renderable (Path V2 n) b)
+    => TrailLike (QDiagram b V2 n Any) where
+  trailLike = strokeP . trailLike
+
+-- | 'stroke'' specialised to 'Path'.
+strokeP' :: (TypeableFloat n, Renderable (Path V2 n) b, IsName a)
+    => StrokeOpts a -> Path V2 n -> QDiagram b V2 n Any
+strokeP' opts path
+  | null (pLines ^. _Wrapped') = mkP pLoops
   | null (pLoops ^. _Wrapped') = mkP pLines
-  | otherwise                   = mkP pLines <> mkP pLoops
+  | otherwise                  = mkP pLines <> mkP pLoops
   where
     (pLines,pLoops) = partitionPath (isLine . unLoc) path
     mkP p
@@ -187,116 +214,114 @@ stroke' opts path
          )
          (Query $ Any . flip (runFillRule (opts^.queryFillRule)) p)
 
+-- | 'stroke'' specialised to 'Path'.
+strokePath' :: (TypeableFloat n, Renderable (Path V2 n) b, IsName a)
+    => StrokeOpts a -> Path V2 n -> QDiagram b V2 n Any
+strokePath' = strokeP'
 
--- | A composition of 'stroke' and 'pathFromTrail' for conveniently
---   converting a trail directly into a diagram.
---
---   Note that a bug in GHC 7.0.1 causes a context stack overflow when
---   inferring the type of 'stroke' and hence of @strokeTrail@ as well.
---   The solution is to give a type signature to expressions involving
---   @strokeTrail@, or (recommended) upgrade GHC (the bug is fixed in 7.0.2
---   onwards).
-strokeTrail :: (Renderable (Path R2) b) => Trail R2 -> Diagram b R2
+-- | 'stroke' specialised to 'Trail'.
+strokeTrail :: (TypeableFloat n, Renderable (Path V2 n) b)
+            => Trail V2 n -> QDiagram b V2 n Any
 strokeTrail = stroke . pathFromTrail
 
--- | Deprecated synonym for 'strokeTrail'.
-strokeT :: (Renderable (Path R2) b) => Trail R2 -> Diagram b R2
+-- | 'stroke' specialised to 'Trail'.
+strokeT :: (TypeableFloat n, Renderable (Path V2 n) b)
+        => Trail V2 n -> QDiagram b V2 n Any
 strokeT = strokeTrail
 
 -- | A composition of 'stroke'' and 'pathFromTrail' for conveniently
 --   converting a trail directly into a diagram.
-strokeTrail' :: (Renderable (Path R2) b, IsName a)
-             => StrokeOpts a -> Trail R2 -> Diagram b R2
+strokeTrail' :: (TypeableFloat n, Renderable (Path V2 n) b, IsName a)
+             => StrokeOpts a -> Trail V2 n -> QDiagram b V2 n Any
 strokeTrail' opts = stroke' opts . pathFromTrail
 
 -- | Deprecated synonym for 'strokeTrail''.
-strokeT' :: (Renderable (Path R2) b, IsName a)
-         => StrokeOpts a -> Trail R2 -> Diagram b R2
+strokeT' :: (TypeableFloat n, Renderable (Path V2 n) b, IsName a)
+         => StrokeOpts a -> Trail V2 n -> QDiagram b V2 n Any
 strokeT' = strokeTrail'
 
 -- | A composition of 'strokeT' and 'wrapLine' for conveniently
 --   converting a line directly into a diagram.
-strokeLine :: (Renderable (Path R2) b) => Trail' Line R2 -> Diagram b R2
+strokeLine :: (TypeableFloat n, Renderable (Path V2 n) b)
+           => Trail' Line V2 n -> QDiagram b V2 n Any
 strokeLine = strokeT . wrapLine
 
 -- | A composition of 'strokeT' and 'wrapLoop' for conveniently
 --   converting a loop directly into a diagram.
-strokeLoop :: (Renderable (Path R2) b) => Trail' Loop R2 -> Diagram b R2
+strokeLoop :: (TypeableFloat n, Renderable (Path V2 n) b)
+           => Trail' Loop V2 n -> QDiagram b V2 n Any
 strokeLoop = strokeT . wrapLoop
 
 -- | A convenience function for converting a @Located Trail@ directly
 --   into a diagram; @strokeLocTrail = stroke . trailLike@.
-strokeLocTrail :: (Renderable (Path R2) b) => Located (Trail R2) -> Diagram b R2
-strokeLocTrail = stroke . trailLike
+strokeLocTrail :: (TypeableFloat n, Renderable (Path V2 n) b)
+               => Located (Trail V2 n) -> QDiagram b V2 n Any
+strokeLocTrail = strokeP . trailLike
 
 -- | Deprecated synonym for 'strokeLocTrail'.
-strokeLocT :: (Renderable (Path R2) b) => Located (Trail R2) -> Diagram b R2
+strokeLocT :: (TypeableFloat n, Renderable (Path V2 n) b)
+           => Located (Trail V2 n) -> QDiagram b V2 n Any
 strokeLocT = strokeLocTrail
 
 -- | A convenience function for converting a @Located@ line directly
 --   into a diagram; @strokeLocLine = stroke . trailLike . mapLoc wrapLine@.
-strokeLocLine :: (Renderable (Path R2) b) => Located (Trail' Line R2) -> Diagram b R2
-strokeLocLine = stroke . trailLike . mapLoc wrapLine
+strokeLocLine :: (TypeableFloat n, Renderable (Path V2 n) b)
+              => Located (Trail' Line V2 n) -> QDiagram b V2 n Any
+strokeLocLine = strokeP . trailLike . mapLoc wrapLine
 
 -- | A convenience function for converting a @Located@ loop directly
 --   into a diagram; @strokeLocLoop = stroke . trailLike . mapLoc wrapLoop@.
-strokeLocLoop :: (Renderable (Path R2) b) => Located (Trail' Loop R2) -> Diagram b R2
-strokeLocLoop = stroke . trailLike . mapLoc wrapLoop
+strokeLocLoop :: (TypeableFloat n, Renderable (Path V2 n) b)
+              => Located (Trail' Loop V2 n) -> QDiagram b V2 n Any
+strokeLocLoop = strokeP . trailLike . mapLoc wrapLoop
 
 ------------------------------------------------------------
 --  Inside/outside testing
 ------------------------------------------------------------
 
-
-
-runFillRule :: FillRule -> P2 -> Path R2 -> Bool
+runFillRule :: RealFloat n => FillRule -> Point V2 n -> Path V2 n -> Bool
 runFillRule Winding = isInsideWinding
 runFillRule EvenOdd = isInsideEvenOdd
 
-newtype FillRuleA = FillRuleA (Last FillRule)
-  deriving (Typeable, Semigroup, Show)
-instance AttributeClass FillRuleA
-
-instance Default FillRuleA where
-  def = FillRuleA $ Last $ def
-
 -- | Extract the fill rule from a 'FillRuleA' attribute.
-getFillRule :: FillRuleA -> FillRule
-getFillRule (FillRuleA (Last r)) = r
+getFillRule :: FillRule -> FillRule
+getFillRule = id
 
 -- | Specify the fill rule that should be used for determining which
 --   points are inside a path.
 fillRule :: HasStyle a => FillRule -> a -> a
-fillRule = applyAttr . FillRuleA . Last
+fillRule = applyAttr
 
-cross :: R2 -> R2 -> Double
-cross (coords -> x :& y) (coords -> x' :& y') = x * y' - y * x'
+-- | Lens onto the fill rule of a style.
+_fillRule :: Lens' (Style V2 n) FillRule
+_fillRule = atAttr . non def
 
 -- XXX link to more info on this
 
--- | Test whether the given point is inside the given (closed) path,
+-- | Test whether the given point is inside the given path,
 --   by testing whether the point's /winding number/ is nonzero. Note
---   that @False@ is /always/ returned for /open/ paths, regardless of
---   the winding number.
-isInsideWinding :: P2 -> Path R2 -> Bool
+--   that @False@ is /always/ returned for paths consisting of lines
+--   (as opposed to loops), regardless of the winding number.
+isInsideWinding :: RealFloat n => Point V2 n -> Path V2 n -> Bool
 isInsideWinding p = (/= 0) . crossings p
 
--- | Test whether the given point is inside the given (closed) path,
+-- | Test whether the given point is inside the given path,
 --   by testing whether a ray extending from the point in the positive
 --   x direction crosses the path an even (outside) or odd (inside)
 --   number of times.  Note that @False@ is /always/ returned for
---   /open/ paths, regardless of the number of crossings.
-isInsideEvenOdd :: P2 -> Path R2 -> Bool
+--   paths consisting of lines (as opposed to loops), regardless of
+--   the number of crossings.
+isInsideEvenOdd :: RealFloat n => Point V2 n -> Path V2 n -> Bool
 isInsideEvenOdd p = odd . crossings p
 
 -- | Compute the sum of /signed/ crossings of a path as we travel in the
 --   positive x direction from a given point.
-crossings :: P2 -> Path R2 -> Int
+crossings :: RealFloat n => Point V2 n -> Path V2 n -> Int
 crossings p = F.sum . map (trailCrossings p) . op Path
 
 -- | Compute the sum of signed crossings of a trail starting from the
 --   given point in the positive x direction.
-trailCrossings :: P2 -> Located (Trail R2) -> Int
+trailCrossings :: RealFloat n => Point V2 n -> Located (Trail V2 n) -> Int
 
   -- non-loop trails have no inside or outside, so don't contribute crossings
 trailCrossings _ t | not (isLoop (unLoc t)) = 0
@@ -309,10 +334,10 @@ trailCrossings p@(unp2 -> (x,y)) tr
       | by <= y && ay > y && isLeft a b < 0 = -1
       | otherwise                           =  0
 
-    test c@(FCubic (unp2 -> x1@(_,x1y))
-                   (unp2 -> c1@(_,c1y))
-                   (unp2 -> c2@(_,c2y))
-                   (unp2 -> x2@(_,x2y))
+    test c@(FCubic (P x1@(V2 _ x1y))
+                   (P c1@(V2 _ c1y))
+                   (P c2@(V2 _ c2y))
+                   (P x2@(V2 _ x2y))
            ) =
         sum . map testT $ ts
       where ts = filter (liftA2 (&&) (>=0) (<=1))
@@ -323,15 +348,15 @@ trailCrossings p@(unp2 -> (x,y)) tr
             testT t = let (unp2 -> (px,_)) = c `atParam` t
                       in  if px > x then signFromDerivAt t else 0
             signFromDerivAt t =
-              let (dx,dy) = (3*t*t) *^ ((-1)*^x1 ^+^ 3*^c1 ^-^ 3*^c2 ^+^ x2)
-                        ^+^ (2*t)   *^ (3*^x1 ^-^ 6*^c1 ^+^ 3*^c2)
-                        ^+^            ((-3)*^x1 ^+^ 3*^c1)
-                  ang = atan2 dy dx
+              let v =  (3*t*t) *^ ((-1)*^x1 ^+^ 3*^c1 ^-^ 3*^c2 ^+^ x2)
+                   ^+^ (2*t)   *^ (3*^x1 ^-^ 6*^c1 ^+^ 3*^c2)
+                   ^+^            ((-3)*^x1 ^+^ 3*^c1)
+                  ang = v ^. _theta . rad
               in  case () of _ | 0      < ang && ang < tau/2 && t < 1 ->  1
                                | -tau/2 < ang && ang < 0     && t > 0 -> -1
                                | otherwise                            ->  0
 
-    isLeft a b = cross (b .-. a) (p .-. a)
+    isLeft a b = cross2 (b .-. a) (p .-. a)
 
 ------------------------------------------------------------
 --  Clipping  ----------------------------------------------
@@ -342,25 +367,36 @@ trailCrossings p@(unp2 -> (x,y)) tr
 --   concatenation, so applying multiple clipping paths is sensible.
 --   The clipping region is the intersection of all the applied
 --   clipping paths.
-newtype Clip = Clip [Path R2]
+newtype Clip n = Clip [Path V2 n]
   deriving (Typeable, Semigroup)
 
 makeWrapped ''Clip
 
-instance AttributeClass Clip
+instance Typeable n => AttributeClass (Clip n)
 
-type instance V Clip = R2
+instance AsEmpty (Clip n) where
+  _Empty = _Clip . _Empty
 
-instance Transformable Clip where
+type instance V (Clip n) = V2
+type instance N (Clip n) = n
+
+instance (OrderedField n) => Transformable (Clip n) where
   transform t (Clip ps) = Clip (transform t ps)
 
+_Clip :: Iso (Clip n) (Clip n') [Path V2 n] [Path V2 n']
+_Clip = _Wrapped
+
+-- | Lens onto the Clip in a style. An empty list means no clipping.
+_clip :: (Typeable n, OrderedField n) => Lens' (Style V2 n) [Path V2 n]
+_clip = atTAttr . non' _Empty . _Clip
+
 -- | Clip a diagram by the given path:
 --
 --   * Only the parts of the diagram which lie in the interior of the
 --     path will be drawn.
 --
 --   * The envelope of the diagram is unaffected.
-clipBy :: (HasStyle a, V a ~ R2) => Path R2 -> a -> a
+clipBy :: (HasStyle a, V a ~ V2, N a ~ n, TypeableFloat n) => Path V2 n -> a -> a
 clipBy = applyTAttr . Clip . (:[])
 
 -- | Clip a diagram to the given path setting its envelope to the
@@ -368,7 +404,8 @@ clipBy = applyTAttr . Clip . (:[])
 --   trace consists of those parts of the original diagram's trace
 --   which fall within the clipping path, or parts of the path's trace
 --   within the original diagram.
-clipTo :: (Renderable (Path R2) b) => Path R2 ->  Diagram b R2 ->  Diagram b R2
+clipTo :: (TypeableFloat n, Renderable (Path V2 n) b)
+  => Path V2 n -> QDiagram b V2 n Any -> QDiagram b V2 n Any
 clipTo p d = setTrace intersectionTrace . toEnvelope $ clipBy p d
   where
     envP = appEnvelope . getEnvelope $ p
@@ -376,8 +413,8 @@ clipTo p d = setTrace intersectionTrace . toEnvelope $ clipBy p d
     toEnvelope = case (envP, envD) of
       (Just eP, Just eD) -> setEnvelope . mkEnvelope $ \v -> min (eP v) (eD v)
       (_, _)             -> id
-    intersectionTrace = Trace intersections
-    intersections pt v =
+    intersectionTrace = Trace traceIntersections
+    traceIntersections pt v =
         -- on boundary of d, inside p
         onSortedList (filter pInside) (appTrace (getTrace d) pt v) <>
         -- or on boundary of p, inside d
@@ -388,5 +425,44 @@ clipTo p d = setTrace intersectionTrace . toEnvelope $ clipBy p d
 
 -- | Clip a diagram to the clip path taking the envelope and trace of the clip
 --   path.
-clipped :: (Renderable (Path R2) b) => Path R2 ->  Diagram b R2 ->  Diagram b R2
-clipped p = (withTrace p) . (withEnvelope p) . (clipBy p)
+clipped :: (TypeableFloat n, Renderable (Path V2 n) b)
+  => Path V2 n -> QDiagram b V2 n Any -> QDiagram b V2 n Any
+clipped p = withTrace p . withEnvelope p . clipBy p
+
+------------------------------------------------------------
+--  Intersections  -----------------------------------------
+------------------------------------------------------------
+
+-- | Find the intersect points of two objects that can be converted to a path.
+intersectPoints :: (InSpace V2 n t, SameSpace t s, ToPath t, ToPath s, OrderedField n)
+  => t -> s -> [P2 n]
+intersectPoints = intersectPoints' 1e-10
+
+-- | Find the intersect points of two objects that can be converted to a path
+--   within the given tolerance.
+intersectPoints' :: (InSpace V2 n t, SameSpace t s, ToPath t, ToPath s, OrderedField n)
+  => n -> t -> s -> [P2 n]
+intersectPoints' eps t s = intersectPointsP' eps (toPath t) (toPath s)
+
+-- | Compute the intersect points between two paths.
+intersectPointsP :: OrderedField n => Path V2 n -> Path V2 n -> [P2 n]
+intersectPointsP = intersectPointsP' 1e-10
+
+-- | Compute the intersect points between two paths within given tolerance.
+intersectPointsP' :: OrderedField n => n -> Path V2 n -> Path V2 n -> [P2 n]
+intersectPointsP' eps as bs = do
+  a <- pathTrails as
+  b <- pathTrails bs
+  intersectPointsT' eps a b
+
+-- | Compute the intersect points between two located trails.
+intersectPointsT :: OrderedField n => Located (Trail V2 n) -> Located (Trail V2 n) -> [P2 n]
+intersectPointsT = intersectPointsT' 1e-10
+
+-- | Compute the intersect points between two located trails within the given
+--   tolerance.
+intersectPointsT' :: OrderedField n => n -> Located (Trail V2 n) -> Located (Trail V2 n) -> [P2 n]
+intersectPointsT' eps as bs = do
+  a <- fixTrail as
+  b <- fixTrail bs
+  intersectPointsS' eps a b
