@@ -1,9 +1,10 @@
-{-# LANGUAGE DeriveFunctor             #-}
-{-# LANGUAGE ExistentialQuantification #-}
-{-# LANGUAGE ScopedTypeVariables       #-}
-{-# LANGUAGE TemplateHaskell           #-}
-{-# LANGUAGE TypeFamilies              #-}
-{-# LANGUAGE ViewPatterns              #-}
+{-# LANGUAGE CPP       #-}
+{-# LANGUAGE DeriveFunctor       #-}
+{-# LANGUAGE FlexibleContexts    #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeFamilies        #-}
+{-# LANGUAGE ViewPatterns        #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -46,20 +47,19 @@ module Diagrams.TwoD.Polygons(
     ) where
 
 import           Control.Lens            (Lens', generateSignatures, lensRules,
-                                          makeLensesWith, (.~), (^.), view)
+                                          makeLensesWith, view, (.~), (^.))
 import           Control.Monad           (forM, liftM)
 import           Control.Monad.ST        (ST, runST)
 import           Data.Array.ST           (STUArray, newArray, readArray,
                                           writeArray)
+import           Data.Default.Class
 import           Data.List               (maximumBy, minimumBy)
 import           Data.Maybe              (catMaybes)
+#if __GLASGOW_HASKELL__ < 710
 import           Data.Monoid             (mconcat, mempty)
+#endif
 import           Data.Ord                (comparing)
 
-import           Data.AffineSpace        ((.+^), (.-.))
-import           Data.Default.Class
-import           Data.VectorSpace
-
 import           Diagrams.Angle
 import           Diagrams.Core
 import           Diagrams.Located
@@ -72,8 +72,12 @@ import           Diagrams.TwoD.Types
 import           Diagrams.TwoD.Vector    (leftTurn, unitX, unitY, unit_Y)
 import           Diagrams.Util           (tau, ( # ))
 
+import           Linear.Affine
+import           Linear.Metric
+import           Linear.Vector
+
 -- | Method used to determine the vertices of a polygon.
-data PolyType = PolyPolar [Angle] [Double]
+data PolyType n = PolyPolar [Angle n] [n]
                 -- ^ A \"polar\" polygon.
                 --
                 --   * The first argument is a list of /central/
@@ -90,21 +94,21 @@ data PolyType = PolyPolar [Angle] [Double]
                 --   circle) can be constructed using a second
                 --   argument of @(repeat r)@.
 
-              | PolySides [Angle] [Double]
+              | PolySides [Angle n] [n]
                 -- ^ A polygon determined by the distance between
-                --   successive vertices and the angles formed by
-                --   each three successive vertices.  In other
+                --   successive vertices and the external angles formed
+                --   by each three successive vertices. In other
                 --   words, a polygon specified by \"turtle
                 --   graphics\": go straight ahead x1 units; turn by
-                --   angle a1; go straght ahead x2 units; turn by
-                --   angle a2; etc. The polygon will be centered at
-                --   the /centroid/ of its vertices.
+                --   external angle a1; go straight ahead x2 units; turn by
+                --   external angle a2; etc. The polygon will be centered
+                --   at the /centroid/ of its vertices.
                 --
                 --   * The first argument is a list of /vertex/
-                --     /angles/, giving the angle at each vertex
+                --     /angles/, giving the external angle at each vertex
                 --     from the previous vertex to the next.  The
-                --     first angle in the list is the angle at the
-                --     /second/ vertex; the first edge always starts
+                --     first angle in the list is the external angle at
+                --     the /second/ vertex; the first edge always starts
                 --     out heading in the positive y direction from
                 --     the first vertex.
                 --
@@ -115,54 +119,54 @@ data PolyType = PolyPolar [Angle] [Double]
                 --   angles and /n-1/ edge lengths.  Extra angles or
                 --   lengths are ignored.
 
-              | PolyRegular Int Double
+              | PolyRegular Int n
                 -- ^ A regular polygon with the given number of
                 --   sides (first argument) and the given radius
                 --   (second argument).
 
 -- | Determine how a polygon should be oriented.
-data PolyOrientation = NoOrient     -- ^ No special orientation; the first
-                                    --   vertex will be at (1,0).
-                                    --   This is the default.
-                     | OrientH      -- ^ Orient /horizontally/, so the
-                                    --   bottommost edge is parallel to
-                                    --   the x-axis.
-                     | OrientV      -- ^ Orient /vertically/, so the
-                                    --   leftmost edge is parallel to the
-                                    --   y-axis.
-                     | OrientTo R2  -- ^ Orient so some edge is
-                                    --   /facing/ /in/ /the/ /direction/
-                                    --   /of/, that is, perpendicular
-                                    --   to, the given vector.
-  deriving (Eq, Ord, Show, Read)
+data PolyOrientation n = NoOrient        -- ^ No special orientation; the first
+                                         --   vertex will be at (1,0).
+                                         --   This is the default.
+                       | OrientH         -- ^ Orient /horizontally/, so the
+                                         --   bottommost edge is parallel to
+                                         --   the x-axis.
+                       | OrientV         -- ^ Orient /vertically/, so the
+                                         --   leftmost edge is parallel to the
+                                         --   y-axis.
+                       | OrientTo (V2 n) -- ^ Orient so some edge is
+                                         --   /facing/ /in/ /the/ /direction/
+                                         --   /of/, that is, perpendicular
+                                         --   to, the given vector.
+                       deriving (Eq, Ord, Show, Read)
 
 -- | Options for specifying a polygon.
-data PolygonOpts = PolygonOpts
-                   { _polyType   :: PolyType
-                   , _polyOrient :: PolyOrientation
-                   , _polyCenter :: P2
+data PolygonOpts n = PolygonOpts
+                   { _polyType   :: PolyType n
+                   , _polyOrient :: PolyOrientation n
+                   , _polyCenter :: Point V2 n
                    }
 
 makeLensesWith (generateSignatures .~ False $ lensRules) ''PolygonOpts
 
 -- | Specification for the polygon's vertices.
-polyType :: Lens' PolygonOpts PolyType
+polyType :: Lens' (PolygonOpts n) (PolyType n)
 
 -- | Should a rotation be applied to the polygon in order to orient it in a
 --   particular way?
-polyOrient :: Lens' PolygonOpts PolyOrientation
+polyOrient :: Lens' (PolygonOpts n) (PolyOrientation n)
 
 -- | Should a translation be applied to the polygon in order to place the center
 --   at a particular location?
-polyCenter :: Lens' PolygonOpts P2
+polyCenter :: Lens' (PolygonOpts n) (Point V2 n)
 
 -- | The default polygon is a regular pentagon of radius 1, centered
 --   at the origin, aligned to the x-axis.
-instance Default PolygonOpts where
+instance Num n => Default (PolygonOpts n) where
     def = PolygonOpts (PolyRegular 5 1) OrientH origin
 
 -- | Generate a polygon.  See 'PolygonOpts' for more information.
-polyTrail :: PolygonOpts -> Located (Trail R2)
+polyTrail :: OrderedField n => PolygonOpts n -> Located (Trail V2 n)
 polyTrail po = transform ori tr
     where
         tr = case po^.polyType of
@@ -176,12 +180,12 @@ polyTrail po = transform ori tr
             NoOrient     -> mempty
 
 -- | Generate the polygon described by the given options.
-polygon :: (TrailLike t, V t ~ R2) => PolygonOpts -> t
+polygon :: (InSpace V2 n t, TrailLike t, OrderedField n) => PolygonOpts n -> t
 polygon = trailLike . polyTrail
 
 -- | Generate the located trail of a polygon specified by polar data
 --   (central angles and radii). See 'PolyPolar'.
-polyPolarTrail :: [Angle] -> [Double] -> Located (Trail R2)
+polyPolarTrail :: OrderedField n =>  [Angle n] -> [n] -> Located (Trail V2 n)
 polyPolarTrail [] _ = emptyTrail `at` origin
 polyPolarTrail _ [] = emptyTrail `at` origin
 polyPolarTrail ans (r:rs) = tr `at` p1
@@ -190,52 +194,52 @@ polyPolarTrail ans (r:rs) = tr `at` p1
     tr = closeTrail . trailFromVertices $
            zipWith
              (\a l -> rotate a . scale l $ p2 (1,0))
-             (scanl (^+^) zeroV ans)
+             (scanl (^+^) zero ans)
              (r:rs)
 
 -- | Generate the vertices of a polygon specified by side length and
 --   angles, and a starting point for the trail such that the origin
 --   is at the centroid of the vertices.  See 'PolySides'.
-polySidesTrail :: [Angle] -> [Double] -> Located (Trail R2)
+polySidesTrail :: OrderedField n =>  [Angle n] -> [n] -> Located (Trail V2 n)
 polySidesTrail ans ls = tr `at` (centroid ps # scale (-1))
   where
-    ans'    = scanl (^+^) zeroV ans
+    ans'    = scanl (^+^) zero ans
     offsets = zipWith rotate ans' (map (unitY ^*) ls)
     ps      = scanl (.+^) origin offsets
     tr      = closeTrail . trailFromOffsets $ offsets
 
 -- | Generate the vertices of a regular polygon.  See 'PolyRegular'.
-polyRegularTrail :: Int -> Double -> Located (Trail R2)
+polyRegularTrail :: OrderedField n =>  Int -> n -> Located (Trail V2 n)
 polyRegularTrail n r = polyPolarTrail
-                         (take (n-1) . repeat $ fullTurn ^/ fromIntegral n)
+                         (replicate (n - 1) $ fullTurn ^/ fromIntegral n)
                          (repeat r)
 
 -- | Generate a transformation to orient a trail.  @orient v t@
 --   generates the smallest rotation such that one of the segments
 --   adjacent to the vertex furthest in the direction of @v@ is
 --   perpendicular to @v@.
-orient :: R2 -> Located (Trail R2) -> T2
+orient :: OrderedField n => V2 n -> Located (Trail V2 n) -> Transformation V2 n
 orient v = orientPoints v . trailVertices
 
-orientPoints :: R2 -> [P2] -> T2
+orientPoints :: OrderedField n => V2 n -> [Point V2 n] -> Transformation V2 n
 orientPoints v xs = rotation a
   where
     (n1,x,n2) = maximumBy (comparing (distAlong v . sndOf3))
                   (zip3 (tail (cycle xs)) xs (last xs : init xs))
-    distAlong w ((.-. origin) -> p) = signum (w <.> p) * magnitude (project w p)
+    distAlong w ((.-. origin) -> p) = signum (w `dot` p) * norm (project w p)
     sndOf3 (_,b,_) = b
-    a :: Angle
+    -- a :: Angle (Scalar v)
     a = minimumBy (comparing $ abs . view rad)
         . map (angleFromNormal . (.-. x)) $ [n1,n2]
-    v' = normalized v
-    angleFromNormal :: R2 -> Angle
+    v' = signorm v
+    -- angleFromNormal :: v -> Angle (Scalar v)
     angleFromNormal o
       | leftTurn o' v' = phi
-      | otherwise      = negateV phi
+      | otherwise      = negated phi
       where
-        o' = normalized o
-        theta = acos (v' <.> o')
-        phi :: Angle
+        o' = signorm o
+        theta = acos (v' `dot` o')
+        -- phi :: Angle (Scalar v)
         phi
           | theta <= tau/4 = tau/4 - theta @@ rad
           | otherwise      = theta - tau/4 @@ rad
@@ -275,10 +279,10 @@ orbits f n = runST genOrbits
     markRho :: Int -> STUArray s Int Bool -> ST s [Int]
     markRho i marks = do
       isMarked <- readArray marks i
-      case isMarked of
-        True  -> return []
-        False -> writeArray marks i True >>
-                 liftM (i:) (markRho (f_n i) marks)
+      if isMarked
+        then return []
+        else writeArray marks i True >>
+               liftM (i:) (markRho (f_n i) marks)
 
     splitParts :: [Int] -> [GraphPart Int]
     splitParts tr = hair ++ cyc
@@ -317,13 +321,13 @@ data StarOpts = StarFun (Int -> Int)
 -- | Create a generalized /star/ /polygon/.  The 'StarOpts' are used
 --   to determine in which order the given vertices should be
 --   connected.  The intention is that the second argument of type
---   @[P2]@ could be generated by a call to 'polygon', 'regPoly', or
+--   @[Point v]@ could be generated by a call to 'polygon', 'regPoly', or
 --   the like, since a list of vertices is 'TrailLike'.  But of course
---   the list can be generated any way you like.  A @'Path' 'R2'@ is
+--   the list can be generated any way you like.  A @'Path' 'v'@ is
 --   returned (instead of any 'TrailLike') because the resulting path
 --   may have more than one component, for example if the vertices are
 --   to be connected in several disjoint cycles.
-star :: StarOpts -> [P2] -> Path R2
+star :: OrderedField n => StarOpts -> [Point V2 n] -> Path V2 n
 star sOpts vs = graphToPath $ mkGraph f vs
   where f = case sOpts of
               StarFun g  -> g
