@@ -45,6 +45,10 @@ module Diagrams.TwoD.Shapes
        , roundedRect'
        ) where
 
+import           Control.Lens            (makeLenses, op, (&), (.~), (<>~), (^.))
+import           Data.Default.Class
+import           Data.Semigroup
+
 import           Diagrams.Core
 
 import           Diagrams.Angle
@@ -57,12 +61,9 @@ import           Diagrams.TwoD.Arc
 import           Diagrams.TwoD.Polygons
 import           Diagrams.TwoD.Transform
 import           Diagrams.TwoD.Types
-
+import           Diagrams.TwoD.Vector
 import           Diagrams.Util
 
-import           Control.Lens            (makeLenses, op, (&), (.~), (^.))
-import           Data.Default.Class
-import           Data.Semigroup
 
 -- | Create a centered horizontal (L-R) line of the given length.
 --
@@ -70,8 +71,8 @@ import           Data.Semigroup
 --
 --   > hruleEx = vcat' (with & sep .~ 0.2) (map hrule [1..5])
 --   >         # centerXY # pad 1.1
-hrule :: (TrailLike t, V t ~ R2) => Double -> t
-hrule d = trailLike $ trailFromSegments [straight $ r2 (d, 0)] `at` (p2 (-d/2,0))
+hrule :: (InSpace V2 n t, TrailLike t) => n -> t
+hrule d = trailLike $ trailFromSegments [straight $ r2 (d, 0)] `at` p2 (-d/2,0)
 
 -- | Create a centered vertical (T-B) line of the given length.
 --
@@ -79,14 +80,14 @@ hrule d = trailLike $ trailFromSegments [straight $ r2 (d, 0)] `at` (p2 (-d/2,0)
 --
 --   > vruleEx = hcat' (with & sep .~ 0.2) (map vrule [1, 1.2 .. 2])
 --   >         # centerXY # pad 1.1
-vrule :: (TrailLike t, V t ~ R2) => Double -> t
-vrule d = trailLike $ trailFromSegments [straight $ r2 (0, (-d))] `at` (p2 (0,d/2))
+vrule :: (InSpace V2 n t, TrailLike t) => n -> t
+vrule d = trailLike $ trailFromSegments [straight $ r2 (0, -d)] `at` p2 (0,d/2)
 
 -- | A square with its center at the origin and sides of length 1,
 --   oriented parallel to the axes.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_unitSquareEx.svg#diagram=unitSquareEx&width=100>>
-unitSquare :: (TrailLike t, V t ~ R2) => t
+unitSquare :: (InSpace V2 n t, TrailLike t, OrderedField n) => t
 unitSquare = polygon (def & polyType   .~ PolyRegular 4 (sqrt 2 / 2)
                           & polyOrient .~ OrientH)
 
@@ -96,7 +97,7 @@ unitSquare = polygon (def & polyType   .~ PolyRegular 4 (sqrt 2 / 2)
 --   length, oriented parallel to the axes.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_squareEx.svg#diagram=squareEx&width=200>>
-square :: (TrailLike t, Transformable t, V t ~ R2) => Double -> t
+square :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 square d = rect d d
 
 -- > squareEx = hcat' (with & sep .~ 0.5) [square 1, square 2, square 3]
@@ -106,7 +107,7 @@ square d = rect d d
 --   @h@, centered at the origin.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_rectEx.svg#diagram=rectEx&width=150>>
-rect :: (TrailLike t, Transformable t, V t ~ R2) => Double -> Double -> t
+rect :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> n -> t
 rect w h = trailLike . head . op Path $ unitSquare # scaleX w # scaleY h
 
 -- > rectEx = rect 1 0.7 # pad 1.1
@@ -138,7 +139,7 @@ rect w h = trailLike . head . op Path $ unitSquare # scaleX w # scaleY h
 --   polygons of a given /radius/).
 --
 --   The polygon will be oriented with one edge parallel to the x-axis.
-regPoly :: (TrailLike t, V t ~ R2) => Int -> Double -> t
+regPoly :: (InSpace V2 n t, TrailLike t, OrderedField n) => Int -> n -> t
 regPoly n l = polygon (def & polyType .~
                                PolySides
                                  (repeat (1/fromIntegral n @@ turn))
@@ -158,90 +159,90 @@ regPoly n l = polygon (def & polyType .~
 -- > dodecagonEx  = shapeEx dodecagon
 
 -- | A synonym for 'triangle', provided for backwards compatibility.
-eqTriangle :: (TrailLike t, V t ~ R2) => Double -> t
+eqTriangle :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 eqTriangle = triangle
 
 -- | An equilateral triangle, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_triangleEx.svg#diagram=triangleEx&width=100>>
-triangle :: (TrailLike t, V t ~ R2) => Double -> t
+triangle :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 triangle = regPoly 3
 
 -- | A regular pentagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_pentagonEx.svg#diagram=pentagonEx&width=100>>
-pentagon :: (TrailLike t, V t ~ R2) => Double -> t
+pentagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 pentagon = regPoly 5
 
 -- | A regular hexagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_hexagonEx.svg#diagram=hexagonEx&width=100>>
-hexagon :: (TrailLike t, V t ~ R2) => Double -> t
+hexagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 hexagon = regPoly 6
 
 -- | A regular heptagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_heptagonEx.svg#diagram=heptagonEx&width=100>>
-heptagon :: (TrailLike t, V t ~ R2) => Double -> t
+heptagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 heptagon = regPoly 7
 
 -- | A synonym for 'heptagon'.  It is, however, completely inferior,
 --   being a base admixture of the Latin /septum/ (seven) and the
 --   Greek γωνία (angle).
-septagon :: (TrailLike t, V t ~ R2) => Double -> t
+septagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 septagon = heptagon
 
 -- | A regular octagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_octagonEx.svg#diagram=octagonEx&width=100>>
-octagon :: (TrailLike t, V t ~ R2) => Double -> t
+octagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 octagon = regPoly 8
 
 -- | A regular nonagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_nonagonEx.svg#diagram=nonagonEx&width=100>>
-nonagon :: (TrailLike t, V t ~ R2) => Double -> t
+nonagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 nonagon = regPoly 9
 
 -- | A regular decagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_decagonEx.svg#diagram=decagonEx&width=100>>
-decagon :: (TrailLike t, V t ~ R2) => Double -> t
+decagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 decagon = regPoly 10
 
 -- | A regular hendecagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_hendecagonEx.svg#diagram=hendecagonEx&width=100>>
-hendecagon :: (TrailLike t, V t ~ R2) => Double -> t
+hendecagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 hendecagon = regPoly 11
 
 -- | A regular dodecagon, with sides of the given length and base
 --   parallel to the x-axis.
 --
 --   <<diagrams/src_Diagrams_TwoD_Shapes_dodecagonEx.svg#diagram=dodecagonEx&width=100>>
-dodecagon :: (TrailLike t, V t ~ R2) => Double -> t
+dodecagon :: (InSpace V2 n t, TrailLike t, OrderedField n) => n -> t
 dodecagon = regPoly 12
 
 ------------------------------------------------------------
 --  Other shapes  ------------------------------------------
 ------------------------------------------------------------
-data RoundedRectOpts = RoundedRectOpts { _radiusTL :: Double
-                                       , _radiusTR :: Double
-                                       , _radiusBL :: Double
-                                       , _radiusBR :: Double
-                                       }
+data RoundedRectOpts d = RoundedRectOpts { _radiusTL :: d
+                                         , _radiusTR :: d
+                                         , _radiusBL :: d
+                                         , _radiusBR :: d
+                                         }
 
 makeLenses ''RoundedRectOpts
 
-instance Default RoundedRectOpts where
+instance (Num d) => Default (RoundedRectOpts d) where
   def = RoundedRectOpts 0 0 0 0
 
 -- | @roundedRect w h r@ generates a closed trail, or closed path
@@ -265,7 +266,7 @@ instance Default RoundedRectOpts where
 --   >                                & radiusBR .~ 0.1)
 --   >   ]
 
-roundedRect :: (TrailLike t, V t ~ R2) => Double -> Double -> Double -> t
+roundedRect :: (InSpace V2 n t, TrailLike t, RealFloat n) => n -> n -> n -> t
 roundedRect w h r = roundedRect' w h (def & radiusTL .~ r
                                           & radiusBR .~ r
                                           & radiusTR .~ r
@@ -275,10 +276,10 @@ roundedRect w h r = roundedRect' w h (def & radiusTL .~ r
 --   each corner indivually, using @RoundedRectOpts@. The default corner radius is 0.
 --   Each radius can also be negative, which results in the curves being reversed
 --   to be inward instead of outward.
-roundedRect' :: (TrailLike t, V t ~ R2) => Double -> Double -> RoundedRectOpts -> t
+roundedRect' :: (InSpace V2 n t, TrailLike t, RealFloat n) => n -> n -> RoundedRectOpts n -> t
 roundedRect' w h opts
    = trailLike
-   . (`at` (p2 (w/2, abs rBR - h/2)))
+   . (`at` p2 (w/2, abs rBR - h/2))
    . wrapTrail
    . glueLine
    $ seg (0, h - abs rTR - abs rBR)
@@ -310,6 +311,8 @@ roundedRect' w h opts
                                 else r
         sign n = if n < 0 then -1 else 1
         mkCorner k r | r == 0    = mempty
-                     | r < 0     = doArc 3 2
+                     | r < 0     = doArc 3 (-1)
                      | otherwise = doArc 0 1
-                     where doArc d d' = arc' r ((k+d)/4 @@ turn) ((k+d')/4 @@ turn)
+                     where
+                       doArc d s =
+                           arc' r (xDir & _theta <>~ ((k+d)/4 @@ turn)) (s/4 @@ turn)
