@@ -1,12 +1,14 @@
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE RankNTypes            #-}
 {-# LANGUAGE TypeFamilies          #-}
+{-# LANGUAGE TypeOperators         #-}
 {-# LANGUAGE ViewPatterns          #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.TwoD.Transform
--- Copyright   :  (c) 2011 diagrams-lib team (see LICENSE)
+-- Copyright   :  (c) 2011-2015 diagrams-lib team (see LICENSE)
 -- License     :  BSD-style (see LICENSE)
 -- Maintainer  :  diagrams-discuss@googlegroups.com
 --
@@ -18,10 +20,12 @@
 
 module Diagrams.TwoD.Transform
        (
+         T2
          -- * Rotation
-         rotation, rotate, rotateBy
+       , rotation, rotate, rotateBy, rotated
 
-       , rotationAbout, rotateAbout
+       , rotationAround, rotateAround
+       , rotationTo, rotateTo
 
          -- * Scaling
        , scalingX, scaleX
@@ -45,35 +49,32 @@ module Diagrams.TwoD.Transform
        , shearingX, shearX
        , shearingY, shearY
 
-         -- * Utilities
-       , onBasis
        ) where
 
-import           Diagrams.Core
-import qualified Diagrams.Core.Transform as T
-
 import           Diagrams.Angle
+import           Diagrams.Core
+import           Diagrams.Core.Transform
+import           Diagrams.Direction
 import           Diagrams.Transform
-import           Diagrams.TwoD.Size      (height, width)
 import           Diagrams.TwoD.Types
-import           Diagrams.TwoD.Vector    (direction)
-import           Diagrams.Coordinates
+import           Diagrams.TwoD.Vector
 
-import           Data.AdditiveGroup
-import           Data.AffineSpace
+import           Control.Lens            hiding (at, transform)
 import           Data.Semigroup
-import           Control.Lens            (review, (^.))
+
+import           Linear.Affine
+import           Linear.Vector
 
 -- Rotation ------------------------------------------------
 
 -- | Create a transformation which performs a rotation about the local
 --   origin by the given angle.  See also 'rotate'.
-rotation :: Angle -> T2
-rotation ang = fromLinear r (linv r)
+rotation :: Floating n => Angle n -> T2 n
+rotation theta = fromLinear r (linv r)
   where
-    r            = rot theta <-> rot (-theta)
-    theta    = ang^.rad
-    rot th (coords -> x :& y) = (cos th * x - sin th * y) ^& (sin th * x + cos th * y)
+    r               = rot theta <-> rot (negated theta)
+    rot th (V2 x y) = V2 (cosA th * x - sinA th * y)
+                         (sinA th * x + cosA th * y)
 
 -- | Rotate about the local origin by the given angle. Positive angles
 --   correspond to counterclockwise rotation, negative to
@@ -88,164 +89,195 @@ rotation ang = fromLinear r (linv r)
 --   will yield an error since GHC cannot figure out which sort of
 --   angle you want to use.  In this common situation you can use
 --   'rotateBy', which interprets its argument as a number of turns.
-rotate :: (Transformable t, V t ~ R2) => Angle -> t -> t
+rotate :: (InSpace V2 n t, Transformable t, Floating n) => Angle n -> t -> t
 rotate = transform . rotation
 
 -- | A synonym for 'rotate', interpreting its argument in units of
 -- turns; it can be more convenient to write @rotateBy (1\/4)@ than
 -- @'rotate' (1\/4 \@\@ 'turn')@.
-rotateBy :: (Transformable t, V t ~ R2) => Double -> t -> t
+rotateBy :: (InSpace V2 n t, Transformable t, Floating n) => n -> t -> t
 rotateBy = transform . rotation . review turn
 
+-- | Use an 'Angle' to make an 'Iso' between an object
+--   rotated and unrotated. This us useful for performing actions
+--   'under' a rotation:
+--
+-- @
+-- under (rotated t) f = rotate (negated t) . f . rotate t
+-- rotated t ## a      = rotate t a
+-- a ^. rotated t      = rotate (-t) a
+-- over (rotated t) f  = rotate t . f . rotate (negated t)
+-- @
+rotated :: (InSpace V2 n a, Floating n, SameSpace a b, Transformable a, Transformable b)
+        => Angle n -> Iso a b a b
+rotated = transformed . rotation
+
 -- | @rotationAbout p@ is a rotation about the point @p@ (instead of
 --   around the local origin).
-rotationAbout :: P2 -> Angle -> T2
-rotationAbout p angle = conjugate (translation (origin .-. p)) (rotation angle)
+rotationAround :: Floating n => P2 n -> Angle n -> T2 n
+rotationAround p theta =
+  conjugate (translation (origin .-. p)) (rotation theta)
 
 -- | @rotateAbout p@ is like 'rotate', except it rotates around the
 --   point @p@ instead of around the local origin.
-rotateAbout :: (Transformable t, V t ~ R2) => P2 -> Angle -> t -> t
-rotateAbout p angle = rotate angle `under` translation (origin .-. p)
+rotateAround :: (InSpace V2 n t, Transformable t, Floating n)
+             => P2 n -> Angle n -> t -> t
+rotateAround p theta = transform (rotationAround p theta)
+
+-- | The rotation that aligns the x-axis with the given direction.
+rotationTo :: OrderedField n => Direction V2 n -> T2 n
+rotationTo (view _Dir -> V2 x y) = rotation (atan2A' y x)
+
+-- | Rotate around the local origin such that the x axis aligns with the
+--   given direction.
+rotateTo :: (InSpace V2 n t, OrderedField n, Transformable t) => Direction V2 n -> t -> t
+rotateTo = transform . rotationTo
 
 -- Scaling -------------------------------------------------
 
 -- | Construct a transformation which scales by the given factor in
 --   the x (horizontal) direction.
-scalingX :: Double -> T2
-scalingX c = fromLinear s s
-  where s = (\(R2 x y) -> R2 (x*c) y) <-> (\(R2 x y) -> R2 (x/c) y)
+scalingX :: (Additive v, R1 v, Fractional n) => n -> Transformation v n
+scalingX c = fromSymmetric $ (_x *~ c) <-> (_x //~ c)
 
 -- | Scale a diagram by the given factor in the x (horizontal)
 --   direction.  To scale uniformly, use 'scale'.
-scaleX :: (Transformable t, V t ~ R2) => Double -> t -> t
+scaleX :: (InSpace v n t, R2 v, Fractional n, Transformable t) => n -> t -> t
 scaleX = transform . scalingX
 
 -- | Construct a transformation which scales by the given factor in
 --   the y (vertical) direction.
-scalingY :: Double -> T2
-scalingY c = fromLinear s s
-  where s = (\(R2 x y) -> R2 x (y*c)) <-> (\(R2 x y) -> R2 x (y/c))
+scalingY :: (Additive v, R2 v, Fractional n) => n -> Transformation v n
+scalingY c = fromSymmetric $ (_y *~ c) <-> (_y //~ c)
 
 -- | Scale a diagram by the given factor in the y (vertical)
 --   direction.  To scale uniformly, use 'scale'.
-scaleY :: (Transformable t, V t ~ R2) => Double -> t -> t
+scaleY :: (InSpace v n t, R2 v, Fractional n, Transformable t)
+  => n -> t -> t
 scaleY = transform . scalingY
 
 -- | @scaleToX w@ scales a diagram in the x (horizontal) direction by
 --   whatever factor required to make its width @w@.  @scaleToX@
 --   should not be applied to diagrams with a width of 0, such as
 --   'vrule'.
-scaleToX :: (Enveloped t, Transformable t, V t ~ R2) => Double -> t -> t
-scaleToX w d = scaleX (w / width d) d
+scaleToX :: (InSpace v n t, R2 v, Enveloped t, Transformable t) => n -> t -> t
+scaleToX w d = scaleX (w / diameter unitX d) d
 
 -- | @scaleToY h@ scales a diagram in the y (vertical) direction by
 --   whatever factor required to make its height @h@.  @scaleToY@
 --   should not be applied to diagrams with a height of 0, such as
 --   'hrule'.
-scaleToY :: (Enveloped t, Transformable t, V t ~ R2) => Double -> t -> t
-scaleToY h d = scaleY (h / height d) d
+scaleToY :: (InSpace v n t, R2 v, Enveloped t, Transformable t) => n -> t -> t
+scaleToY h d = scaleY (h / diameter unitY d) d
 
 -- | @scaleUToX w@ scales a diagram /uniformly/ by whatever factor
 --   required to make its width @w@.  @scaleUToX@ should not be
 --   applied to diagrams with a width of 0, such as 'vrule'.
-scaleUToX :: (Enveloped t, Transformable t, V t ~ R2) => Double -> t -> t
-scaleUToX w d = scale (w / width d) d
+scaleUToX :: (InSpace v n t, R1 v, Enveloped t, Transformable t) => n -> t -> t
+scaleUToX w d = scale (w / diameter unitX d) d
 
 -- | @scaleUToY h@ scales a diagram /uniformly/ by whatever factor
 --   required to make its height @h@.  @scaleUToY@ should not be applied
 --   to diagrams with a height of 0, such as 'hrule'.
-scaleUToY :: (Enveloped t, Transformable t, V t ~ R2) => Double -> t -> t
-scaleUToY h d = scale (h / height d) d
+scaleUToY :: (InSpace v n t, R2 v, Enveloped t, Transformable t) => n -> t -> t
+scaleUToY h d = scale (h / diameter unitY d) d
 
 -- Translation ---------------------------------------------
 
 -- | Construct a transformation which translates by the given distance
 --   in the x (horizontal) direction.
-translationX :: Double -> T2
-translationX x = translation (x ^& 0)
+translationX :: (Additive v, R1 v, Num n) => n -> Transformation v n
+translationX x = translation (zero & _x .~ x)
 
 -- | Translate a diagram by the given distance in the x (horizontal)
 --   direction.
-translateX :: (Transformable t, V t ~ R2) => Double -> t -> t
+translateX :: (InSpace v n t, R1 v, Transformable t) => n -> t -> t
 translateX = transform . translationX
 
 -- | Construct a transformation which translates by the given distance
 --   in the y (vertical) direction.
-translationY :: Double -> T2
-translationY y = translation (0 ^& y)
+translationY :: (Additive v, R2 v, Num n) => n -> Transformation v n
+translationY y = translation (zero & _y .~ y)
 
 -- | Translate a diagram by the given distance in the y (vertical)
 --   direction.
-translateY :: (Transformable t, V t ~ R2) => Double -> t -> t
+translateY :: (InSpace v n t, R2 v, Transformable t)
+  => n -> t -> t
 translateY = transform . translationY
 
 -- Reflection ----------------------------------------------
 
 -- | Construct a transformation which flips a diagram from left to
 --   right, i.e. sends the point (x,y) to (-x,y).
-reflectionX :: T2
-reflectionX = scalingX (-1)
+reflectionX :: (Additive v, R1 v, Num n) => Transformation v n
+reflectionX = fromSymmetric $ (_x *~ (-1)) <-> (_x *~ (-1))
 
 -- | Flip a diagram from left to right, i.e. send the point (x,y) to
 --   (-x,y).
-reflectX :: (Transformable t, V t ~ R2) => t -> t
+reflectX :: (InSpace v n t, R1 v, Transformable t) => t -> t
 reflectX = transform reflectionX
 
 -- | Construct a transformation which flips a diagram from top to
 --   bottom, i.e. sends the point (x,y) to (x,-y).
-reflectionY :: T2
-reflectionY = scalingY (-1)
+reflectionY :: (Additive v, R2 v, Num n) => Transformation v n
+reflectionY = fromSymmetric $ (_y *~ (-1)) <-> (_y *~ (-1))
 
 -- | Flip a diagram from top to bottom, i.e. send the point (x,y) to
 --   (x,-y).
-reflectY :: (Transformable t, V t ~ R2) => t -> t
+reflectY :: (InSpace v n t, R2 v, Transformable t) => t -> t
 reflectY = transform reflectionY
 
--- | @reflectionAbout p v@ is a reflection in the line determined by
---   the point @p@ and vector @v@.
-reflectionAbout :: P2 -> R2 -> T2
-reflectionAbout p v =
-  conjugate (rotation (negateV $ direction v) <> translation (origin .-. p))
+-- | @reflectionAbout p d@ is a reflection in the line determined by
+--   the point @p@ and direction @d@.
+reflectionAbout :: OrderedField n => P2 n -> Direction V2 n -> T2 n
+reflectionAbout p d =
+  conjugate (rotationTo d <> translation (origin .-. p))
             reflectionY
 
--- | @reflectAbout p v@ reflects a diagram in the line determined by
---   the point @p@ and the vector @v@.
-reflectAbout :: (Transformable t, V t ~ R2) => P2 -> R2 -> t -> t
+-- | @reflectAbout p d@ reflects a diagram in the line determined by
+--   the point @p@ and direction @d@.
+reflectAbout :: (InSpace V2 n t, OrderedField n, Transformable t)
+             => P2 n -> Direction V2 n -> t -> t
 reflectAbout p v = transform (reflectionAbout p v)
 
 -- Shears --------------------------------------------------
 
+-- auxiliary functions for shearingX/shearingY
+sh :: (n -> n -> n -> n) -> (n -> n -> n -> n) -> n -> V2 n -> V2 n
+sh f g k (V2 x y) = V2 (f k x y) (g k x y)
+
+sh' :: (n -> n -> n -> n) -> (n -> n -> n -> n) -> n -> V2 n -> V2 n
+sh' f g k = swap . sh f g k . swap
+
+swap :: V2 n -> V2 n
+swap (V2 x y) = V2 y x
+{-# INLINE swap #-}
+
 -- | @shearingX d@ is the linear transformation which is the identity on
 --   y coordinates and sends @(0,1)@ to @(d,1)@.
-shearingX :: Double -> T2
-shearingX d = fromLinear (sh d  <-> sh (-d))
-                         (sh' d <-> sh' (-d))
-  where sh  k (R2 x y) = R2 (x+k*y) y
-        sh' k          = swap . sh k . swap
-        swap (R2 x y)  = R2 y x
+shearingX :: Num n => n -> T2 n
+shearingX d = fromLinear (sh f g d  <-> sh f g (-d))
+                         (sh' f g d <-> sh' f g (-d))
+  where
+    f k x y = x + k*y
+    g _ _ y = y
 
 -- | @shearX d@ performs a shear in the x-direction which sends
 --   @(0,1)@ to @(d,1)@.
-shearX :: (Transformable t, V t ~ R2) => Double -> t -> t
+shearX :: (InSpace V2 n t, Transformable t) => n -> t -> t
 shearX = transform . shearingX
 
 -- | @shearingY d@ is the linear transformation which is the identity on
 --   x coordinates and sends @(1,0)@ to @(1,d)@.
-shearingY :: Double -> T2
-shearingY d = fromLinear (sh d  <-> sh (-d))
-                         (sh' d <-> sh' (-d))
-  where sh  k (R2 x y) = R2 x (y+k*x)
-        sh' k          = swap . sh k . swap
-        swap (R2 x y)  = R2 y x
+shearingY :: Num n => n -> T2 n
+shearingY d = fromLinear (sh f g d  <-> sh f g (-d))
+                         (sh' f g d <-> sh' f g (-d))
+        where
+          f _ x _ = x
+          g k x y = y + k*x
 
 -- | @shearY d@ performs a shear in the y-direction which sends
 --   @(1,0)@ to @(1,d)@.
-shearY :: (Transformable t, V t ~ R2) => Double -> t -> t
+shearY :: (InSpace V2 n t, Transformable t) => n -> t -> t
 shearY = transform . shearingY
 
--- | Get the matrix equivalent of the linear transform,
---   (as a pair of columns) and the translation vector.  This
---   is mostly useful for implementing backends.
-onBasis :: Transformation R2 -> ((R2, R2), R2)
-onBasis t = ((x, y), v)
-  where ((x:y:[]), v) = T.onBasis t
